@@ -1,5 +1,5 @@
 package Terse;
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 use 5.006;
 use strict;
 use warnings;
@@ -145,10 +145,12 @@ sub logger {
 
 sub logError {
 	my ($self, $message, $status) = @_;
-	$self->raiseError($message, $status);
+	$self->{_application} 
+		? $self->response->raiseError($message, $status) 
+		: $self->raiseError($message, $status);
 	$message = { message => $message } if (!ref $message);
-	$message = $self->{_application}->logError($message, $status)
-		if ($self->{_application} && $self->{_application}->can('logError'));
+	$message = $self->{_application}->_logError($message, $status)
+		if ($self->{_application} && $self->{_application}->can('_logError'));
 	$self->{_logger}->err($message) if $self->{_logger};
 	return $self;
 }
@@ -156,13 +158,14 @@ sub logError {
 sub logInfo {
 	my ($self, $message) = @_;
 	$message = { message => $message } if (!ref $message);
-	$message = $self->{_application}->logInfo($message)
-		if ($self->{_application} && $self->{_application}->can('logInfo'));
+	$message = $self->{_application}->_logInfo($message)
+		if ($self->{_application} && $self->{_application}->can('_logInfo'));
 	$self->{_logger}->info($message) if $self->{_logger};
 }
 
 sub raiseError {
 	my ($self, $message, $code) = @_;
+	return $self->response->raiseError($message, $code) if $self->{_application};
 	$self->{error} = \1;
 	if ((reftype($message) || '') eq 'ARRAY') {
 		push @{$self->{errors}}, @{$message};
@@ -236,6 +239,7 @@ sub AUTOLOAD : lvalue {
 	die "illegal key name, must be of $validname form\n$AUTOLOAD" unless $key;
 	my $miss = Want::want('REF OBJECT') ? {} : '';
 	my $retval = $_[0]->{$key};
+	return $retval->(@_) if (ref $retval eq 'CODE');
 	my $isBool = Want::want('SCALAR BOOL') && ((reftype($retval) // '') eq 'SCALAR');
 	$retval = $$retval if $isBool;
 	$_[0]->{$key} = $_[1] // $retval // $miss;
@@ -253,7 +257,7 @@ Terse - lightweight JSON APIs.
 
 =head1 VERSION
 
-Version 0.01
+Version 0.04
 
 =cut
 
@@ -271,16 +275,19 @@ Version 0.01
 
 	sub hello_world {
 		my ($self, $t) = @_;
+	
+		if ($t->params->throw_error) {
+			$t->logError('throw 500 error which is also logged', 500);
+			return;
+		}
 
 		$t->response->hello = "world";
 	}
 
-
 	.... MyAPI.psgi ...
 
 	use Terse;
 	use MyAPI;
-
 	our $api = MyAPI->new();
 
 	sub {
@@ -288,6 +295,14 @@ Version 0.01
 		Terse->run(
 			plack_env => $env,
 			application => $api,
+			logger => Terse->new(
+				info => sub { 
+					say "info log line: " . $_[1]->{message};
+				}, 
+				err => sub { 
+					say "err log line: " . $_[1]->{message}; 
+				} 
+			)
 		);
 	};
 
@@ -299,6 +314,139 @@ Version 0.01
 	# {"authenticated":1,"error":false,"errors":[],"hello":"world","status_code":200}
 	GET http://localhost:5000/?req=hello_world&not=1 
 	# {"authenticated":0,"error":true,"errors":["Invalid request"],"status_code":400}
+	GET http://localhost:5000/?req=hello_world&throw_error=1 
+	# {"authenticated":1,"error":true,"errors":["throw 500 error which is also logged"],"status_code":500}
+
+=cut
+
+=head1 Description
+
+Alot of the inspiration, and some code, for this module came from L<JSONP> - which is a module to quickly build JSON/JSONP web services, providing also some syntactic sugar acting a bit like a sort of DSL (domain specific language) for JSON. ( thanks Anselmo Canfora L<ACANFORA>! )
+
+There are several key differences between Terse and L<JSONP>, the main being Terse uses Plack and not CGI. Terse also makes it simpler to provision the data which should be returned from the API (and what should not), finally it adds logging support.
+
+=cut
+
+=head1 Methods
+
+=cut
+
+=head2 new
+
+Instantiate a new Terse object.
+
+	my $object = Terse->new(%params);
+
+=cut
+
+=head2 run
+
+Run terse as a plack application.
+
+	Terse->run(
+		login => 'login',
+		logout => 'logout',
+		auth => 'auth',
+		insecure_session => 0,
+		application => Terse->new(
+			auth => sub { ... },
+			login => sub { ... },
+			logout => sub { ... }
+		),
+		plack_env => $env
+	);
+
+The "application" does not need to be a "Terse application", the only requirment is it implements the auth, login and logout methods (go crazy!).
+
+=cut
+
+=head2 params
+
+Retrieve params for the request.
+
+	$terse->params;
+
+=cut
+
+=head2 request
+
+Returns the Plack::Request.
+
+	$terse->request;
+
+=cut
+
+=head2 session
+
+Retrieve current session data, set in your auth or login methods
+
+	$terse->session;
+
+=cut
+
+=head2 response
+
+Set the response body data.
+
+	$terse->response->foo = { ... };
+
+=cut
+
+=head2 logger
+
+Set or Retrieve the logger for the application.
+
+	$terse->logger($logger);
+	$terse->logger->info();
+	$terse->logger->err();
+
+=cut
+
+=head2 logError
+
+Log and raise an error message.
+
+	$terse->logError('this is an error message', 404);
+
+=cut
+
+=head2 logInfo
+
+Log an info message.
+
+	$terse->logInfo('this an info message');
+
+=cut
+
+=head2 raiseError
+
+Raise an error message.
+
+	$terse->raiseError('this is an error message', 404);
+
+=cut
+
+=head2 graft
+
+Decode a JSON string.
+
+	$terse->response->graft('config', "{...}");
+
+=cut
+
+=head2 pretty
+
+Set JSON to pretty print mode.
+
+	$terse->pretty(1);
+
+=cut
+
+=head2 serialize
+
+Encode a perl struct as a JSON string.
+
+	$terse->serialize({ ... });
 
 =cut
 
