@@ -11,4 +11,8 @@ sub hello_world {
 	$_[1]->response->hello = "world";
 }
 
+sub error {
+	$_[1]->logError('test an error', 500);
+}
+
 1;
